@@ -69,6 +69,7 @@ GenEvent::GenEvent(const GenEvent&e) {
         GenEventData tdata;
         e.write_data(tdata);
         read_data(tdata);
+        m_run_info = e.m_run_info;
     }
 }
 
@@ -89,6 +90,7 @@ GenEvent& GenEvent::operator=(const GenEvent& e) {
         GenEventData tdata;
         e.write_data(tdata);
         read_data(tdata);
+        m_run_info = e.m_run_info;
     }
     return *this;
 }
@@ -161,7 +163,7 @@ void GenEvent::remove_particle(GenParticlePtr p) {
             }
         }
 
-        std::sort(changed_attributes.begin(),changed_attributes.end(), [](const std::pair< int, std::shared_ptr<Attribute> > &a, const std::pair< int, std::shared_ptr<Attribute> > &b){ return a.first < b.first; });
+        std::sort(changed_attributes.begin(),changed_attributes.end(), [](const std::pair< int, std::shared_ptr<Attribute> > &a, const std::pair< int, std::shared_ptr<Attribute> > &b) { return a.first < b.first; });
         for ( const auto& val: changed_attributes ) {
             vt1.second.erase(val.first);
             vt1.second[val.first-1] = val.second;
@@ -231,7 +233,7 @@ void GenEvent::remove_vertex(GenVertexPtr v) {
         }
 
         std::reverse(changed_attributes.begin(),changed_attributes.end());
-        std::sort(changed_attributes.begin(),changed_attributes.end(),[](const std::pair< int, std::shared_ptr<Attribute> > &a, const std::pair< int, std::shared_ptr<Attribute> > &b){ return a.first > b.first; });
+        std::sort(changed_attributes.begin(),changed_attributes.end(),[](const std::pair< int, std::shared_ptr<Attribute> > &a, const std::pair< int, std::shared_ptr<Attribute> > &b) { return a.first > b.first; });
         for ( const auto& val: changed_attributes ) {
             vt1.second.erase(val.first);
             vt1.second[val.first+1] = val.second;
@@ -247,8 +249,9 @@ void GenEvent::remove_vertex(GenVertexPtr v) {
     v->m_event = nullptr;
     v->m_id    = 0;
 }
-/// @todo This looks dangerously similar to the recusive event traversel that we forbade in the
-///       Core library due to wories about generator dependence
+/* This looks dangerously similar to the recusive event traversel that we forbade in the
+       Core library due to wories about generator dependence
+*/
 static bool visit_children(std::map<ConstGenVertexPtr, int>  &a, const ConstGenVertexPtr& v)
 {
     for (const ConstGenParticlePtr& p: v->particles_out()) {
@@ -368,12 +371,12 @@ void GenEvent::add_tree(const std::vector<GenParticlePtr> &parts) {
                     vt1.second[val.first == rootid? 0: val.first + 1] = val.second;
                 }
             }
-            m_rootvertex->set_id(0);
+            m_rootvertex->m_id = 0;
             while ( next != m_vertices.end() ) {
                 ++((*next++)->m_id);
             }
         } else {
-            HEPMC3_WARNING("ReaderAsciiHepMC2: Suspicious looking rootvertex found. Will try to cope.")
+            HEPMC3_WARNING_LEVEL(700,"GenEvent::add_tree Suspicious looking rootvertex found. Will try to cope.")
         }
     }
 
@@ -419,7 +422,7 @@ const FourVector& GenEvent::event_pos() const {
 std::vector<ConstGenParticlePtr> GenEvent::beams(const int status) const {
     if (!status) return std::const_pointer_cast<const GenVertex>(m_rootvertex)->particles_out();
     std::vector<ConstGenParticlePtr> ret;
-    for (auto p: m_rootvertex->particles_out()) if (p->status() == status) ret.emplace_back(p);
+    for (auto& p: m_rootvertex->particles_out()) if (p->status() == status) ret.emplace_back(p);
     return ret;
 }
 
@@ -445,6 +448,13 @@ void GenEvent::shift_position_by(const FourVector & delta) {
 
 bool GenEvent::rotate(const FourVector&  delta)
 {
+    long double cosa = std::cos(delta.x());
+    long double sina = std::sin(delta.x());
+    long double cosb = std::cos(delta.y());
+    long double sinb = std::sin(delta.y());
+    long double cosg = std::cos(delta.z());
+    long double sing = std::sin(delta.z());
+
     for ( auto& p: m_particles)
     {
         const FourVector& mom = p->momentum();
@@ -452,26 +462,16 @@ bool GenEvent::rotate(const FourVector&  delta)
         long double tempY = mom.y();
         long double tempZ = mom.z();
 
-        long double cosa = std::cos(delta.x());
-        long double sina = std::sin(delta.x());
-
         long double tempY_ = cosa*tempY+sina*tempZ;
         long double tempZ_ = -sina*tempY+cosa*tempZ;
         tempY = tempY_;
         tempZ = tempZ_;
 
-
-        long double cosb = std::cos(delta.y());
-        long double sinb = std::sin(delta.y());
-
         long double tempX_ = cosb*tempX-sinb*tempZ;
         tempZ_ = sinb*tempX+cosb*tempZ;
         tempX = tempX_;
         tempZ = tempZ_;
 
-        long double cosg = std::cos(delta.z());
-        long double sing = std::sin(delta.z());
-
         tempX_ = cosg*tempX+sing*tempY;
         tempY_ = -sing*tempX+cosg*tempY;
         tempX = tempX_;
@@ -483,30 +483,22 @@ bool GenEvent::rotate(const FourVector&  delta)
     for (auto& v: m_vertices)
     {
         const FourVector& pos = v->position();
+        if (pos.is_zero()) continue;
+
         long double tempX = pos.x();
         long double tempY = pos.y();
         long double tempZ = pos.z();
 
-        long double cosa = std::cos(delta.x());
-        long double sina = std::sin(delta.x());
-
         long double tempY_ = cosa*tempY+sina*tempZ;
         long double tempZ_ = -sina*tempY+cosa*tempZ;
         tempY = tempY_;
         tempZ = tempZ_;
 
-
-        long double cosb = std::cos(delta.y());
-        long double sinb = std::sin(delta.y());
-
         long double tempX_ = cosb*tempX-sinb*tempZ;
         tempZ_ = sinb*tempX+cosb*tempZ;
         tempX = tempX_;
         tempZ = tempZ_;
 
-        long double cosg = std::cos(delta.z());
-        long double sing = std::sin(delta.z());
-
         tempX_ = cosg*tempX+sing*tempY;
         tempY_ = -sing*tempX+cosg*tempY;
         tempX = tempX_;
@@ -524,7 +516,7 @@ bool GenEvent::reflect(const int axis)
 {
     if ( axis > 3 || axis < 0 )
     {
-        HEPMC3_WARNING("GenEvent::reflect: wrong axis")
+        HEPMC3_WARNING_LEVEL(400,"GenEvent::reflect: wrong axis")
         return false;
     }
     switch (axis)
@@ -554,26 +546,25 @@ bool GenEvent::reflect(const int axis)
 
 bool GenEvent::boost(const FourVector&  delta)
 {
-    double deltalength2d = delta.length2();
-    if (deltalength2d > 1.0)
+    double deltalength2 = delta.length2();
+    if (deltalength2 > 1.0)
     {
-        HEPMC3_WARNING("GenEvent::boost: wrong large boost vector. Will leave event as is.")
+        HEPMC3_WARNING_LEVEL(400,"GenEvent::boost: wrong large boost vector. Will leave event as is.")
         return false;
     }
-    if (std::abs(deltalength2d-1.0) < std::numeric_limits<double>::epsilon())
+    if (std::abs(deltalength2-1.0) < std::numeric_limits<double>::epsilon())
     {
-        HEPMC3_WARNING("GenEvent::boost: too large gamma. Will leave event as is.")
+        HEPMC3_WARNING_LEVEL(400,"GenEvent::boost: too large gamma. Will leave event as is.")
         return false;
     }
-    if (std::abs(deltalength2d) < std::numeric_limits<double>::epsilon())
+    if (std::abs(deltalength2) < std::numeric_limits<double>::epsilon())
     {
-        HEPMC3_WARNING("GenEvent::boost: wrong small boost vector. Will leave event as is.")
+        HEPMC3_WARNING_LEVEL(400,"GenEvent::boost: wrong small boost vector. Will leave event as is.")
         return true;
     }
     long double deltaX = delta.x();
     long double deltaY = delta.y();
     long double deltaZ = delta.z();
-    long double deltalength2 = deltaX*deltaX+deltaY*deltaY+deltaZ*deltaZ;
     long double deltalength = std::sqrt(deltalength2);
     long double gamma = 1.0/std::sqrt(1.0-deltalength2);
 
@@ -586,10 +577,10 @@ bool GenEvent::boost(const FourVector&  delta)
         long double tempZ = mom.z();
         long double tempE = mom.e();
         long double nr = (deltaX*tempX+deltaY*tempY+deltaZ*tempZ)/deltalength;
-
-        tempX+=(deltaX*((gamma-1)*nr/deltalength)-deltaX*(tempE*gamma));
-        tempY+=(deltaY*((gamma-1)*nr/deltalength)-deltaY*(tempE*gamma));
-        tempZ+=(deltaZ*((gamma-1)*nr/deltalength)-deltaZ*(tempE*gamma));
+        long double gfac = (gamma-1)*nr/deltalength-tempE*gamma;
+        tempX+=(deltaX*gfac);
+        tempY+=(deltaY*gfac);
+        tempZ+=(deltaZ*gfac);
         tempE = gamma*(tempE-deltalength*nr);
         FourVector temp(tempX, tempY, tempZ, tempE);
         p->set_momentum(temp);
@@ -676,7 +667,7 @@ void GenEvent::write_data(GenEventData& data) const {
             bool status = vt2.second->to_string(st);
 
             if ( !status ) {
-                HEPMC3_WARNING("GenEvent::write_data: problem serializing attribute: " << vt1.first)
+                HEPMC3_WARNING_LEVEL(300,"GenEvent::write_data: problem serializing attribute: " << vt1.first)
             }
             else {
                 data.attribute_id.emplace_back(vt2.first);
@@ -724,7 +715,10 @@ void GenEvent::read_data(const GenEventData &data) {
         (+-)  --  particle has end vertex
         (-+)  --  particle  has production vertex
         */
-        if ((id1 < 0 && id2 <0) || (id1 > 0 && id2 > 0))   { HEPMC3_WARNING("GenEvent::read_data: wrong link: " << id1 << " " << id2); continue;}
+        if ((id1 < 0 && id2 <0) || (id1 > 0 && id2 > 0))   {
+            HEPMC3_WARNING_LEVEL(600,"GenEvent::read_data: wrong link: " << id1 << " " << id2);
+            continue;
+        }
 
         if ( id1 > 0 ) { m_vertices[ (-id2)-1 ]->add_particle_in ( m_particles[ id1-1 ] ); continue; }
         if ( id1 < 0 ) { m_vertices[ (-id1)-1 ]->add_particle_out( m_particles[ id2-1 ] );   continue; }
@@ -756,16 +750,6 @@ void GenEvent::read_data(const GenEventData &data) {
 // Deprecated functions
 //
 
-void GenEvent::add_particle(GenParticle *p) {
-    add_particle(GenParticlePtr(p));
-}
-
-
-void GenEvent::add_vertex(GenVertex *v) {
-    add_vertex(GenVertexPtr(v));
-}
-
-
 void GenEvent::set_beam_particles(GenParticlePtr p1, GenParticlePtr p2) {
     m_rootvertex->add_particle_out(p1);
     m_rootvertex->add_particle_out(p2);
@@ -774,12 +758,12 @@ void GenEvent::set_beam_particles(GenParticlePtr p1, GenParticlePtr p2) {
 void GenEvent::add_beam_particle(GenParticlePtr p1) {
     if (!p1)
     {
-        HEPMC3_WARNING("Attempting to add an empty particle as beam particle. Ignored.")
+        HEPMC3_WARNING_LEVEL(700,"Attempting to add an empty particle as beam particle. Ignored.")
         return;
     }
     if (p1->in_event() && p1->parent_event() != this)
     {
-        HEPMC3_WARNING("Attempting to add particle from another event. Ignored.")
+        HEPMC3_WARNING_LEVEL(700,"Attempting to add particle from another event. Ignored.")
         return;
     }
     if (p1->production_vertex())  p1->production_vertex()->remove_particle_out(p1);
