@@ -23,9 +23,9 @@
 namespace HepMC3 {
 
 ReaderAsciiHepMC2::ReaderAsciiHepMC2(const std::string& filename):
-    m_file(filename), m_stream(nullptr), m_isstream(false) {
+    m_file(filename), m_isstream(false) {
     if ( !m_file.is_open() ) {
-        HEPMC3_ERROR("ReaderAsciiHepMC2: could not open input file: " << filename )
+        HEPMC3_ERROR_LEVEL(100,"ReaderAsciiHepMC2: could not open input file: " << filename )
     }
     set_run_info(std::make_shared<GenRunInfo>());
     m_event_ghost = new GenEvent();
@@ -35,7 +35,7 @@ ReaderAsciiHepMC2::ReaderAsciiHepMC2(std::istream & stream)
     : m_stream(&stream), m_isstream(true)
 {
     if ( !m_stream->good() ) {
-        HEPMC3_ERROR("ReaderAsciiHepMC2: could not open input stream ")
+        HEPMC3_ERROR_LEVEL(100,"ReaderAsciiHepMC2: could not open input stream ")
     }
     set_run_info(std::make_shared<GenRunInfo>());
     m_event_ghost = new GenEvent();
@@ -45,9 +45,10 @@ ReaderAsciiHepMC2::ReaderAsciiHepMC2(std::shared_ptr<std::istream> s_stream)
     : m_shared_stream(s_stream), m_stream(s_stream.get()), m_isstream(true)
 {
     if ( !m_stream->good() ) {
-        HEPMC3_ERROR("ReaderAsciiHepMC2: could not open input stream ")
+        HEPMC3_ERROR_LEVEL(100,"ReaderAsciiHepMC2: could not open input stream ")
     }
     set_run_info(std::make_shared<GenRunInfo>());
+    m_event_ghost = new GenEvent();
 }
 
 
@@ -55,7 +56,7 @@ ReaderAsciiHepMC2::~ReaderAsciiHepMC2() { if (m_event_ghost) { m_event_ghost->cl
 
 bool ReaderAsciiHepMC2::skip(const int n)
 {
-    std::array<char, 262144> buf;
+    std::array<char, 262144> buf{};
     int nn = n;
     while (!failed()) {
         char peek(0);
@@ -72,7 +73,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
     if ( (!m_file.is_open()) && (!m_isstream) ) return false;
 
     char               peek = 0;
-    std::array<char, 262144> buf;
+    std::array<char, 262144> buf{};
     bool          parsed_event_header            = false;
     bool          is_parsing_successful          = true;
     int           parsing_result                 = 0;
@@ -101,7 +102,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
         if ( strncmp(buf.data(), "HepMC", 5) == 0 ) {
             if ( strncmp(buf.data(), "HepMC::Version", 14) != 0 && strncmp(buf.data(), "HepMC::IO_GenEvent", 18) != 0 )
             {
-                HEPMC3_WARNING("ReaderAsciiHepMC2: found unsupported expression in header. Will close the input.")
+                HEPMC3_WARNING_LEVEL(500,"ReaderAsciiHepMC2: found unsupported expression in header. Will close the input.")
                 std::cout <<buf.data() << std::endl;
                 m_isstream ? m_stream->clear(std::ios::eofbit) : m_file.clear(std::ios::eofbit);
             }
@@ -116,7 +117,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
             parsing_result = parse_event_information(evt, buf.data());
             if (parsing_result < 0) {
                 is_parsing_successful = false;
-                HEPMC3_ERROR("ReaderAsciiHepMC2: HEPMC3_ERROR parsing event information")
+                HEPMC3_ERROR_LEVEL(600,"ReaderAsciiHepMC2: HEPMC3_ERROR parsing event information")
             }
             else {
                 vertices_count = parsing_result;
@@ -135,7 +136,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
         case 'V':
             // If starting new vertex: verify if previous was fully parsed
 
-            /** @bug HepMC2 files produced with Pythia8 are known to have wrong
+            /** HepMC2 files produced with Pythia8 are known to have wrong
                      information about number of particles in vertex. Hence '<' sign */
             if (current_vertex_particles_parsed < current_vertex_particles_count) {
                 is_parsing_successful = false;
@@ -147,7 +148,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
 
             if (parsing_result < 0) {
                 is_parsing_successful = false;
-                HEPMC3_ERROR("ReaderAsciiHepMC2: HEPMC3_ERROR parsing vertex information")
+                HEPMC3_ERROR_LEVEL(600,"ReaderAsciiHepMC2: HEPMC3_ERROR parsing vertex information")
             }
             else {
                 current_vertex_particles_count = parsing_result;
@@ -160,7 +161,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
 
             if (parsing_result < 0) {
                 is_parsing_successful = false;
-                HEPMC3_ERROR("ReaderAsciiHepMC2: HEPMC3_ERROR parsing particle information")
+                HEPMC3_ERROR_LEVEL(600,"ReaderAsciiHepMC2: HEPMC3_ERROR parsing particle information")
             }
             else {
                 ++current_vertex_particles_parsed;
@@ -183,7 +184,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
             is_parsing_successful = parse_xs_info(evt, buf.data());
             break;
         default:
-            HEPMC3_WARNING("ReaderAsciiHepMC2: skipping unrecognised prefix: " << buf[0])
+            HEPMC3_WARNING_LEVEL(500,"ReaderAsciiHepMC2: skipping unrecognised prefix: " << buf[0])
             is_parsing_successful = true;
             break;
         }
@@ -196,20 +197,20 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
     }
 
     // Check if all particles in last vertex were parsed
-    /** @bug HepMC2 files produced with Pythia8 are known to have wrong
+    /** HepMC2 files produced with Pythia8 are known to have wrong
              information about number of particles in vertex. Hence '<' sign */
     if (is_parsing_successful && current_vertex_particles_parsed < current_vertex_particles_count) {
-        HEPMC3_ERROR("ReaderAsciiHepMC2: not all particles parsed")
+        HEPMC3_ERROR_LEVEL(600,"ReaderAsciiHepMC2: not all particles parsed")
         is_parsing_successful = false;
     }
     // Check if all vertices were parsed
     else if (is_parsing_successful && m_vertex_cache.size() != vertices_count) {
-        HEPMC3_ERROR("ReaderAsciiHepMC2: not all vertices parsed")
+        HEPMC3_ERROR_LEVEL(600,"ReaderAsciiHepMC2: not all vertices parsed")
         is_parsing_successful = false;
     }
 
     if ( !is_parsing_successful ) {
-        HEPMC3_ERROR("ReaderAsciiHepMC2: event parsing failed. Returning empty event")
+        HEPMC3_ERROR_LEVEL(600,"ReaderAsciiHepMC2: event parsing failed. Returning empty event")
         HEPMC3_DEBUG(1, "Parsing failed at line:" << std::endl << buf.data())
         evt.clear();
         m_isstream ? m_stream->clear(std::ios::badbit) : m_file.clear(std::ios::badbit);
@@ -219,7 +220,7 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
         run_info()->set_weight_names(std::vector<std::string> {"Default"});
     }
     if (evt.weights().empty()) {
-        HEPMC3_WARNING("ReaderAsciiHepMC2: weights are empty, an event weight 1.0 will be added.")
+        HEPMC3_WARNING_LEVEL(600,"ReaderAsciiHepMC2: weights are empty, an event weight 1.0 will be added.")
         evt.weights().push_back(1.0);
     }
 
@@ -321,7 +322,6 @@ bool ReaderAsciiHepMC2::read_event(GenEvent &evt) {
             }
         }
     }
-
     std::shared_ptr<IntAttribute> signal_process_vertex_barcode = evt.attribute<IntAttribute>("signal_process_vertex");
     if (signal_process_vertex_barcode) {
         int signal_process_vertex_barcode_value = signal_process_vertex_barcode->value();
@@ -582,6 +582,7 @@ int ReaderAsciiHepMC2::parse_particle_information(const char *buf) {
         data_ghost->add_attribute("flows", std::make_shared<VectorIntAttribute>(vectorflows));
     }
     // Set prod_vtx link
+    if (m_vertex_cache.empty()) {HEPMC3_DEBUG(1, "The first particle in event appears before the first vertex"); return -1;}
     if ( end_vtx == m_vertex_barcodes.back() ) {
         m_vertex_cache.back()->add_particle_in(data);
         end_vtx = 0;
@@ -608,7 +609,8 @@ bool ReaderAsciiHepMC2::parse_xs_info(GenEvent &evt, const char *buf) {
 
     if ( !(cursor = strchr(cursor+1, ' ')) ) return false;
     const double xs_err = atof(cursor);
-    const size_t all = m_options.count("disable_pad_cross_sections") ? size_t{1} : std::max(evt.weights().size(),size_t{1});
+    const size_t all = m_options.count("disable_pad_cross_sections") ? size_t{1} :
+                       std::max(evt.weights().size(),size_t{1});
     const double xs_val_dummy = m_options.count("pad_cross_section_value") ? std::strtod(m_options.at("pad_cross_section_value").c_str(),nullptr) : 0.0;
     const double xs_err_dummy = m_options.count("pad_cross_section_error") ? std::strtod(m_options.at("pad_cross_section_error").c_str(),nullptr) : 0.0;
     xs->set_cross_section(std::vector<double>(all,xs_val_dummy), std::vector<double>(all,xs_err_dummy));
