@@ -16,10 +16,10 @@ namespace HepMC3
 {
 
 ReaderHEPEVT::ReaderHEPEVT(const std::string &filename)
-    : m_file(filename), m_stream(nullptr), m_isstream(false)
+    : m_file(filename), m_isstream(false)
 {
     if ( !m_file.is_open() ) {
-        HEPMC3_ERROR("ReaderHEPEVT: could not open input file: " << filename)
+        HEPMC3_ERROR_LEVEL(100,"ReaderHEPEVT: could not open input file: " << filename)
     }
     else
     {
@@ -33,7 +33,7 @@ ReaderHEPEVT::ReaderHEPEVT(std::istream & stream)
     : m_stream(&stream), m_isstream(true)
 {
     if ( !m_stream->good() ) {
-        HEPMC3_ERROR("ReaderHEPEVT: could not open input stream  ")
+        HEPMC3_ERROR_LEVEL(100,"ReaderHEPEVT: could not open input stream  ")
     }
     else
     {
@@ -46,7 +46,7 @@ ReaderHEPEVT::ReaderHEPEVT(std::shared_ptr<std::istream> s_stream)
     : m_shared_stream(s_stream), m_stream(s_stream.get()), m_isstream(true)
 {
     if ( !m_stream->good() ) {
-        HEPMC3_ERROR("ReaderHEPEVT: could not open input stream  ")
+        HEPMC3_ERROR_LEVEL(100,"ReaderHEPEVT: could not open input stream  ")
     }
     else
     {
@@ -58,7 +58,7 @@ ReaderHEPEVT::ReaderHEPEVT(std::shared_ptr<std::istream> s_stream)
 
 bool ReaderHEPEVT::skip(const int n)
 {
-    std::array<char, 262144> buf;
+    std::array<char, 262144> buf{};
     int nn = n;
     while (!failed()) {
         char peek(0);
@@ -75,7 +75,7 @@ bool ReaderHEPEVT::skip(const int n)
 
 bool ReaderHEPEVT::read_hepevt_event_header()
 {
-    std::array<char, 512> buf_e;
+    std::array<char, 512> buf_e{};
     bool eventline = false;
     int m_i = 0;
     int m_p = 0;
@@ -105,11 +105,11 @@ bool ReaderHEPEVT::read_hepevt_event_header()
 
 bool ReaderHEPEVT::read_hepevt_particle(int i)
 {
-    std::array<char, 512> buf_p;
-    std::array<char, 512> buf_v;
-    std::array<int, 6>   intcodes;
-    std::array<double, 5> fltcodes1;
-    std::array<double, 5> fltcodes2;
+    std::array<char, 512> buf_p{};
+    std::array<char, 512> buf_v{};
+    std::array<int, 6>   intcodes{};
+    std::array<double, 5> fltcodes1{};
+    std::array<double, 5> fltcodes2{};
     m_isstream ? m_stream->getline(buf_p.data(),buf_p.size()) : m_file.getline(buf_p.data(),buf_p.size());
     if ( strlen(buf_p.data()) == 0 ) return false;
     if (m_options.count("vertices_positions_are_absent") == 0)
@@ -121,12 +121,21 @@ bool ReaderHEPEVT::read_hepevt_particle(int i)
     std::stringstream st_v(buf_v.data());
     if (m_options.count("vertices_positions_are_absent") == 0)
     {
-        if (!static_cast<bool>(st_p >> intcodes[0] >> intcodes[1] >> intcodes[2] >> intcodes[3] >> intcodes[4] >> intcodes[5] >> fltcodes1[0] >> fltcodes1[1] >> fltcodes1[2] >> fltcodes1[3] >> fltcodes1[4])) { HEPMC3_ERROR("ReaderHEPEVT: HEPMC3_ERROR reading particle momenta");     return false;}
-        if (!static_cast<bool>(st_v >> fltcodes2[0] >> fltcodes2[1] >> fltcodes2[2] >> fltcodes2[3])) { HEPMC3_ERROR("ReaderHEPEVT: HEPMC3_ERROR reading particle vertex");  return false;}
+        if (!static_cast<bool>(st_p >> intcodes[0] >> intcodes[1] >> intcodes[2] >> intcodes[3] >> intcodes[4] >> intcodes[5] >> fltcodes1[0] >> fltcodes1[1] >> fltcodes1[2] >> fltcodes1[3] >> fltcodes1[4])) {
+            HEPMC3_ERROR_LEVEL(100,"ReaderHEPEVT: HEPMC3_ERROR reading particle momenta")
+            return false;
+        }
+        if (!static_cast<bool>(st_v >> fltcodes2[0] >> fltcodes2[1] >> fltcodes2[2] >> fltcodes2[3])) {
+            HEPMC3_ERROR_LEVEL(100,"ReaderHEPEVT: HEPMC3_ERROR reading particle vertex")
+            return false;
+        }
     }
     else
     {
-        if (!static_cast<bool>(st_p>> intcodes[0]>> intcodes[1] >> intcodes[4] >> intcodes[5] >> fltcodes1[0] >> fltcodes1[1] >> fltcodes1[2] >> fltcodes1[4])) {HEPMC3_ERROR("ReaderHEPEVT: HEPMC3_ERROR reading particle momenta");     return false;}
+        if (!static_cast<bool>(st_p>> intcodes[0]>> intcodes[1] >> intcodes[4] >> intcodes[5] >> fltcodes1[0] >> fltcodes1[1] >> fltcodes1[2] >> fltcodes1[4])) {
+            HEPMC3_ERROR_LEVEL(100,"ReaderHEPEVT: HEPMC3_ERROR reading particle momenta")
+            return false;
+        }
         intcodes[2] = 0;//FIXME! This is a feature of this format, but maybe there are better ideas.
         intcodes[3] = 0;//FIXME! This is a feature of this format, but maybe there are better ideas.
         fltcodes1[3] = std::sqrt(fltcodes1[0]*fltcodes1[0]+fltcodes1[1]*fltcodes1[1]+fltcodes1[2]*fltcodes1[2]+fltcodes1[4]*fltcodes1[4]);
